/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaparasite.h
 * Copyright (C) 1998 Jay Cox <jaycox@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_BASE_H_INSIDE__) && !defined (PIKA_BASE_COMPILATION)
#error "Only <libpikabase/pikabase.h> can be included directly."
#endif

#ifndef __PIKA_PARASITE_H__
#define __PIKA_PARASITE_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


/*
 * PIKA_TYPE_PARASITE
 */

#define PIKA_TYPE_PARASITE               (pika_parasite_get_type ())
#define PIKA_VALUE_HOLDS_PARASITE(value) (G_TYPE_CHECK_VALUE_TYPE ((value), PIKA_TYPE_PARASITE))

GType   pika_parasite_get_type           (void) G_GNUC_CONST;


/*
 * PIKA_TYPE_PARAM_PARASITE
 */

#define PIKA_TYPE_PARAM_PARASITE           (pika_param_parasite_get_type ())
#define PIKA_IS_PARAM_SPEC_PARASITE(pspec) (G_TYPE_CHECK_INSTANCE_TYPE ((pspec), PIKA_TYPE_PARAM_PARASITE))

typedef struct _PikaParamSpecParasite PikaParamSpecParasite;

GType        pika_param_parasite_get_type  (void) G_GNUC_CONST;

GParamSpec * pika_param_spec_parasite      (const gchar  *name,
                                            const gchar  *nick,
                                            const gchar  *blurb,
                                            GParamFlags   flags);


#define PIKA_PARASITE_PERSISTENT 1
#define PIKA_PARASITE_UNDOABLE   2

#define PIKA_PARASITE_ATTACH_PARENT     (0x80 << 8)
#define PIKA_PARASITE_PARENT_PERSISTENT (PIKA_PARASITE_PERSISTENT << 8)
#define PIKA_PARASITE_PARENT_UNDOABLE   (PIKA_PARASITE_UNDOABLE << 8)

#define PIKA_PARASITE_ATTACH_GRANDPARENT     (0x80 << 16)
#define PIKA_PARASITE_GRANDPARENT_PERSISTENT (PIKA_PARASITE_PERSISTENT << 16)
#define PIKA_PARASITE_GRANDPARENT_UNDOABLE   (PIKA_PARASITE_UNDOABLE << 16)


/**
 * PikaParasite:
 * @name:  the parasite name, USE A UNIQUE PREFIX
 * @flags: the parasite flags, like save in XCF etc.
 * @size:  the parasite size in bytes
 * @data: (array length=size): the parasite data, the owner os the parasite is responsible
 *   for tracking byte order and internal structure
 **/
struct _PikaParasite
{
  gchar    *name;
  guint32   flags;
  guint32   size;
  gpointer  data;
};


PikaParasite * pika_parasite_new           (const gchar        *name,
                                            guint32             flags,
                                            guint32             size,
                                            gconstpointer       data);
void           pika_parasite_free          (PikaParasite       *parasite);

PikaParasite * pika_parasite_copy          (const PikaParasite *parasite);

gboolean       pika_parasite_compare       (const PikaParasite *a,
                                            const PikaParasite *b);

gboolean       pika_parasite_is_type       (const PikaParasite *parasite,
                                            const gchar        *name);
gboolean       pika_parasite_is_persistent (const PikaParasite *parasite);
gboolean       pika_parasite_is_undoable   (const PikaParasite *parasite);
gboolean       pika_parasite_has_flag      (const PikaParasite *parasite,
                                            gulong              flag);
gulong         pika_parasite_get_flags     (const PikaParasite *parasite);
const gchar  * pika_parasite_get_name      (const PikaParasite *parasite);

gconstpointer  pika_parasite_get_data      (const PikaParasite *parasite,
                                            guint32            *num_bytes);

G_END_DECLS

#endif /* __PIKA_PARASITE_H__ */
